﻿using CommunityToolkit.Mvvm.ComponentModel;
using ContactsApp.Model.Helpers;
using Microsoft.EntityFrameworkCore.Metadata.Internal;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ContactsApp.Model.Entities
{
    public enum AddressCategory
    {
        Home,
        Work,
        Other
    }

    public partial class Address(string city, string street, int houseNumber, int? flatNumber = null, AddressCategory category = AddressCategory.Other) : ObservableDataValidator
    {
        private string _city = city;
        [Required(AllowEmptyStrings = false,
            ErrorMessageResourceName = nameof(ModelResources.AddressCityError),
            ErrorMessageResourceType = typeof(ModelResources))]
        public string City
        {
            get => _city;
            set => SetProperty(ref _city, value, true);
        }

        private string _street = street;
        [Required(AllowEmptyStrings = false,
            ErrorMessageResourceName = nameof(ModelResources.AddressStreetError),
            ErrorMessageResourceType = typeof(ModelResources))]
        public string Street
        {
            get => _street;
            set => SetProperty(ref _street, value, true);
        }

        private int _houseNumber = houseNumber;
        [Required]
        public int HouseNumber
        {
            get => _houseNumber;
            set => SetProperty(ref _houseNumber, value, true);
        }

        [ObservableProperty]
        private int? _flatNumber = flatNumber;

        private AddressCategory _category = category;
        [EnumDataType(typeof(AddressCategory),
            ErrorMessageResourceName = nameof(ModelResources.AddressCategoryError),
            ErrorMessageResourceType = typeof(ModelResources))]
        public AddressCategory Category
        {
            get => _category;
            set => SetProperty(ref _category, value, true);
        }

        public override string ToString()
        {
            return $"{Street} {HouseNumber} {FlatNumber.ToString() ?? string.Empty}\n{City}\n[{Category}]";
        }
    }
}
