﻿using CommunityToolkit.Mvvm.ComponentModel;
using ContactsApp.Model.Helpers;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ContactsApp.Model.Entities
{
    public enum Gender
    {
        Unspecified,
        Female,
        Male,
        Other
    }

    public partial class Contact : ObservableDataValidator
    {
        private string _name;
        [Required(AllowEmptyStrings = false,
            ErrorMessageResourceName = nameof(ModelResources.NameEmptyError),
            ErrorMessageResourceType = typeof(ModelResources))]
        public string Name
        {
            get => _name;
            set => SetProperty(ref _name, value, true);
        }

        [ObservableProperty]
        private DateTimeOffset? _birthDate;

        [ObservableProperty]
        private Gender _gender;

        [ObservableProperty]
        private string? _imageUri;

        public Contact(string name, DateTimeOffset? birthDate = null, Gender gender = Gender.Unspecified, string? imageUri = null)
        {
            _name = name;
            _birthDate = birthDate;
            _gender = gender;
            _imageUri = imageUri;
            Addresses = [];
            Phones = [];
            Emails = [];
            Addresses.CollectionChanged += (s, e) => OnPropertyChanged(nameof(PrimaryAddress));
            Phones.CollectionChanged += (s, e) => OnPropertyChanged(nameof(PrimaryPhone));
            Emails.CollectionChanged += (s, e) => OnPropertyChanged(nameof(PrimaryEmail));
        }

        private void AddressListChanged(object? sender, NotifyCollectionChangedEventArgs e)
        {
        }

        public Phone? PrimaryPhone => Phones.Count > 0 ? Phones[0] : null;
        public Address? PrimaryAddress => Addresses.Count > 0 ? Addresses[0] : null;
        public Email? PrimaryEmail => Emails.Count > 0 ? Emails[0] : null;

        public ObservableCollection<Address> Addresses { get; }
        public ObservableCollection<Phone> Phones { get; }
        public ObservableCollection<Email> Emails { get; }

        public string PrimaryPhoneNumber => Phones.Count > 0 ? Phones[0].Number : string.Empty;
    }
}
