﻿using CommunityToolkit.Mvvm.ComponentModel;
using ContactsApp.Model.Helpers;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ContactsApp.Model.Entities
{
    public enum EmailCategory
    {
        Personal,
        Work,
        Other
    }

    public partial class Email(string address, EmailCategory category = EmailCategory.Other) : ObservableDataValidator
    {
        private string _address = address;

        [Required(AllowEmptyStrings = false, ErrorMessageResourceName = nameof(ModelResources.EmailLengthError))]
        [EmailAddress(ErrorMessageResourceName = nameof(ModelResources.EmailAddressError))]
        public string Address
        {
            get => _address;
            set => SetProperty(ref _address, value, true);
        }

        private EmailCategory _category = category;
        [EnumDataType(typeof(EmailCategory), ErrorMessageResourceName = nameof(ModelResources.EmailCategoryError))]
        public EmailCategory Category
        {
            get => _category;
            set => SetProperty(ref _category, value, true);
        }

        public override string ToString()
        {
            return $"{Address} [{Category}]";
        }
    }
}
