﻿using CommunityToolkit.Mvvm.ComponentModel;
using ContactsApp.Model.Helpers;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ContactsApp.Model.Entities
{
    public enum PhoneCategory
    {
        Home,
        Mobile,
        Work,
        Other
    }

    public partial class Phone(string number, PhoneCategory category = PhoneCategory.Other) : ObservableDataValidator
    {
        private string _number = number;
        [Required(AllowEmptyStrings = false, ErrorMessageResourceName = nameof(ModelResources.PhoneNumberLenghtError))]
        [Phone(ErrorMessageResourceName = nameof(ModelResources.PhoneNumberError))]
        public string Number
        {
            get => _number;
            set => SetProperty(ref _number, value, true);
        }

        private PhoneCategory _category = category;
        [EnumDataType(typeof(PhoneCategory), ErrorMessageResourceName = nameof(ModelResources.PhoneCategoryError))]
        public PhoneCategory Category
        {
            get => _category;
            set => SetProperty(ref _category, value, true);
        }

        public override string ToString()
        {
            return $"{Number} [{Category}]";
        }
    }
}
