﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace ContactsApp.UI
{
    public class RedTextBoxExt : TextBox
    {

        [DllImport("user32.dll")]
        private static extern IntPtr GetWindowDC(IntPtr hwnd);

        struct RECT
        {
            public int left, top, right, bottom;
        }
        struct NCCALCSIZE_PARAMS
        {
            public RECT newWindow;
            public RECT oldWindow;
            public RECT clientWindow;
            IntPtr windowPos;
        }

        private static float clientPadding = 2.0f;
        private static float actualBorderWidth = 4.0f;
        private static Color borderColor = Color.Red;

        public RedTextBoxExt() : base() { }

        protected override void WndProc(ref Message m)
        {
            switch(m.Msg)
            {
                case 0x83://WM_NCCALCSIZE
                    {
                        if (m.WParam == IntPtr.Zero)
                        {
                            RECT rect = (RECT)Marshal.PtrToStructure(m.LParam, typeof(RECT));
                            rect.left += (int)clientPadding;
                            rect.right -= (int)clientPadding;
                            rect.top += (int)clientPadding;
                            rect.bottom -= (int)clientPadding;
                            Marshal.StructureToPtr(rect, m.LParam, false);
                        }
                        else
                        {
                            NCCALCSIZE_PARAMS rects = (NCCALCSIZE_PARAMS)Marshal.PtrToStructure(m.LParam, typeof(NCCALCSIZE_PARAMS));
                            rects.newWindow.left += (int)clientPadding;
                            rects.newWindow.right -= (int)clientPadding;
                            rects.newWindow.top += (int)clientPadding;
                            rects.newWindow.bottom -= (int)clientPadding;
                            Marshal.StructureToPtr(rects, m.LParam, false);
                        }
                        break;
                    }
                case 0x85://WM_NCPAINT
                    {
                        IntPtr wDC = GetWindowDC(Handle);
                        using (Graphics g = Graphics.FromHdc(wDC))
                        {
                            ControlPaint.DrawBorder(g, new Rectangle(0, 0, Size.Width, Size.Height), borderColor, (int)actualBorderWidth, ButtonBorderStyle.Solid,
                                borderColor, (int)actualBorderWidth, ButtonBorderStyle.Solid, borderColor, (int)actualBorderWidth, ButtonBorderStyle.Solid,
                                borderColor, (int)actualBorderWidth, ButtonBorderStyle.Solid);
                        }
                        return;
                    }

            }
            base.WndProc(ref m);
        }
    }
}
