﻿using System;
using System.ComponentModel;
using System.Threading;
using System.Windows.Forms;

namespace WindowsFormsData
{
    public partial class MyForm : Form
    {
        public MyForm()
        {
            InitializeComponent();
            var set = MyFormSettings.Default;
            if (set.autoLoad)
            {
                tbName.Text = set.name;
                tbEmail.Text = set.email;
            }
            lblStatus.Text = MyFormResources.prompt;
        }

        private void EnableForm(bool enabled = true)
        {
            tbName.Enabled = enabled;
            tbEmail.Enabled = enabled;
            btnSave.Enabled = enabled;
            Cursor = enabled ? Cursors.Default : Cursors.WaitCursor;
        }

        private void ClearForm()
        {
            tbName.Clear();
            tbEmail.Clear();
            err.SetError(tbName, string.Empty);
            err.SetError(tbEmail, string.Empty);
            lblStatus.Text = MyFormResources.prompt;
            progress.Visible = false;
            progress.Value = 0;
        }

        private bool ValidateEmail(string emailAddress)
        {
            // Confirm the address is not empty
            if (string.IsNullOrEmpty(emailAddress))
                return false;

            // Confirm there is '@' and '.' somewhere in the address (in that order)
            var atIdx = emailAddress.IndexOf('@');
            return atIdx > -1 && emailAddress.IndexOf('.', atIdx) > -1;
        }

        private void tbValidated(object sender, EventArgs e)
        {
            err.SetError((Control)sender, string.Empty);
        }

        private void tbName_Validating(object sender, CancelEventArgs e)
        {
            if (string.IsNullOrEmpty(tbName.Text))
            {
                e.Cancel = true;
                err.SetError(tbName, MyFormResources.nameError);
            }
        }

        private void tbEmail_Validating(object sender, CancelEventArgs e)
        {
            if (!ValidateEmail(tbEmail.Text))
            {
                e.Cancel = true;
                err.SetError(tbEmail, MyFormResources.emailError);
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            if (ValidateChildren())
            {
                EnableForm(false);
                lblStatus.Text = MyFormResources.progress;
                progress.Visible = true;
                bgwSaving.RunWorkerAsync();
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (bgwSaving.IsBusy)
                bgwSaving.CancelAsync();
            else
                ClearForm();
        }

        private void bgwSaving_DoWork(object sender, DoWorkEventArgs e)
        {
            for (int i = 0; i < 10; i++)
            {
                bgwSaving.ReportProgress(i * 10);
                Thread.Sleep(500);
                if (bgwSaving.CancellationPending)
                {
                    e.Cancel = true;
                    return;
                }
            }
            e.Result = MyFormResources.finishedMessage;
        }

        private void bgwSaving_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progress.Value = e.ProgressPercentage;
        }

        private void bgwSaving_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            string message = MyFormResources.cancelledMessage;
            if (!e.Cancelled)
            {
                message = (string)e.Result;
                progress.Value = 100;
                var set = MyFormSettings.Default;
                set.name = tbName.Text;
                set.email = tbEmail.Text;
                set.Save();
            }
            MessageBox.Show(this, message, MyFormResources.messageTitle);
            ClearForm();
            EnableForm();
        }
    }
}
