﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsData
{
    public partial class MyFormAsync : Form
    {
        public MyFormAsync()
        {
            InitializeComponent();
            var set = MyFormSettings.Default;
            if (set.autoLoad)
            {
                tbName.Text = set.name;
                tbEmail.Text = set.email;
            }
            lblStatus.Text = "Please fill the form";
        }

        private void EnableForm(bool enabled = true)
        {
            tbName.Enabled = enabled;
            tbEmail.Enabled = enabled;
            btnSave.Enabled = enabled;
            Cursor = enabled ? Cursors.Default : Cursors.WaitCursor;
        }

        private void ClearForm()
        {
            tbName.Clear();
            tbEmail.Clear();
            err.SetError(tbName, string.Empty);
            err.SetError(tbEmail, string.Empty);
            lblStatus.Text = "Please fill the form";
            progress.Visible = false;
            progress.Value = 0;
        }

        private bool ValidateEmail(string emailAddress)
        {
            // Confirm the address is not empty
            if (string.IsNullOrEmpty(emailAddress))
                return false;

            // Confirm there is '@' and '.' somewhere in the address (in that order)
            var atIdx = emailAddress.IndexOf('@');
            return atIdx > -1 && emailAddress.IndexOf('.', atIdx) > -1;
        }

        private void tbValidated(object sender, EventArgs e)
        {
            err.SetError((Control)sender, string.Empty);
        }

        private void tbName_Validating(object sender, CancelEventArgs e)
        {
            if (string.IsNullOrEmpty(tbName.Text))
            {
                e.Cancel = true;
                err.SetError(tbName, "Please enter your name!");
            }
        }

        private void tbEmail_Validating(object sender, CancelEventArgs e)
        {
            if (!ValidateEmail(tbEmail.Text))
            {
                e.Cancel = true;
                err.SetError(tbEmail, "Please enter a valid e-mail address!");
            }
        }

        private async void btnSave_Click(object sender, EventArgs e)
        {
            if (ValidateChildren())
            {
                EnableForm(false);
                lblStatus.Text = "Saving...";
                progress.Visible = true;
                var reporter = new Progress<int>(p => progress.Value = p);
                var message = "Cancelled!";
                try
                {
                    using (cancelSource = new())
                    {
                        var cancelToken = cancelSource.Token;
                        //await a task run on thread pool
                        message = await Task.Run(() => DoWork(reporter, cancelToken), cancelToken);
                    }
                    //continuation executed back on the ui thread automatically
                    cancelSource = null;
                    progress.Value = 100;
                    var set = MyFormSettings.Default;
                    set.name = tbName.Text;
                    set.email = tbEmail.Text;
                    set.Save();
                }
                catch (OperationCanceledException ex)
                { }
                MessageBox.Show(this, message, "Result");
                ClearForm();
                EnableForm();
            }
        }

        private async Task<string> DoWork(IProgress<int> reporter, CancellationToken cancelToken)
        {
            for (int i = 0; i < 10; i++)
            {
                await Task.Delay(500, cancelToken); //since DoWork spends most of the time awaiting the delay
                                                    //it could be run on ui thread,
                reporter.Report(i * 10);            //but this demonstrates that progress reporting is dispatched
                                                    //back to the ui thread automatically.
                if (cancelToken.IsCancellationRequested)
                    cancelToken.ThrowIfCancellationRequested();
            }
            return "Finishded!";
        }

        private CancellationTokenSource? cancelSource;

        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (cancelSource != null)
                cancelSource.Cancel();
            else
                ClearForm();
        }
    }
}
